/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>

/**
 * Constantes and conversion maps for the nlohmann::json representation (reader and writer)
 */
namespace armarx::aron::type::rw::json
{
    namespace constantes
    {
        const std::string MAYBE_SLUG = "_ARON_MAYBE";
        const std::string TYPE_SLUG = "_ARON_TYPE";
        const std::string PATH_SLUG = "_ARON_PATH";
        const std::string VERSION_SLUG = "_ARON_VERSION";

        const std::string KEY_SLUG = "_ARON_KEY";
        const std::string VALUE_SLUG = "_ARON_VALUE";
        const std::string MEMBERS_SLUG = "_ARON_MEMBERS";
        const std::string ELEMENTS_SLUG = "_ARON_ELEMENTS";

        const std::string NAME_SLUG = "_ARON_NAME";
        const std::string EXTENDS_SLUG = "_ARON_EXTENDS";
        const std::string ACCEPTED_TYPE_SLUG = "_ARON_ACCEPTED_TYPE";
        const std::string DIMENSIONS_SLUG = "_ARON_DIMESIONS";
        const std::string DATA_SLUG = "_ARON_DATA";
        const std::string USED_TYPE_SLUG = "_ARON_USED_TYPE";
        const std::string TEMPLATES_SLUG = "_ARON_TEMPLATES";
        const std::string TEMPLATE_INSTANTIATIONS_SLUG = "_ARON_TEMPLATE_INSTANTIATION";

        const std::string LIST_TYPENAME_SLUG = "_ARON_LIST";
        const std::string DICT_TYPENAME_SLUG = "_ARON_DICT";
        const std::string OBJECT_TYPENAME_SLUG = "_ARON_OBJECT";
        const std::string TUPLE_TYPENAME_SLUG = "_ARON_TUPLE";
        const std::string PAIR_TYPENAME_SLUG = "_ARON_PAIR";
        const std::string NDARRAY_TYPENAME_SLUG = "_ARON_NDARRAY";
        const std::string IMAGE_TYPENAME_SLUG = "_ARON_IMAGE";
        const std::string MATRIX_TYPENAME_SLUG = "_ARON_MATRIX";
        const std::string ORIENTATION_TYPENAME_SLUG = "_ARON_ORIENTATION";
        const std::string POINT_CLOUD_TYPENAME_SLUG = "_ARON_POINT_CLOUD";
        const std::string POSE_TYPENAME_SLUG = "_ARON_POSE";
        const std::string POSITION_TYPENAME_SLUG = "_ARON_POSITION";
        const std::string QUATERNION_TYPENAME_SLUG = "_ARON_QUATERNION";
        const std::string INT_ENUM_TYPENAME_SLUG = "_ARON_INT_ENUM";
        const std::string INT_TYPENAME_SLUG = "_ARON_INT";
        const std::string LONG_TYPENAME_SLUG = "_ARON_LONG";
        const std::string FLOAT_TYPENAME_SLUG = "_ARON_FLOAT";
        const std::string DOUBLE_TYPENAME_SLUG = "_ARON_DOUBLE";
        const std::string STRING_TYPENAME_SLUG = "_ARON_STRING";
        const std::string BOOL_TYPENAME_SLUG = "_ARON_BOOL";
        const std::string TIME_TYPENAME_SLUG = "_ARON_TIME";
        const std::string ANY_OBJECT_TYPENAME_SLUG = "_ARON_ANY_OBJECT";
    }

    namespace conversion
    {
        const std::map<type::Descriptor, std::string> Descriptor2String = {
            {type::Descriptor::eDict, rw::json::constantes::DICT_TYPENAME_SLUG},
            {type::Descriptor::eList, rw::json::constantes::LIST_TYPENAME_SLUG},
            {type::Descriptor::ePair, rw::json::constantes::PAIR_TYPENAME_SLUG},
            {type::Descriptor::eTuple, rw::json::constantes::TUPLE_TYPENAME_SLUG},
            {type::Descriptor::eObject, rw::json::constantes::OBJECT_TYPENAME_SLUG},
            {type::Descriptor::eNDArray, rw::json::constantes::NDARRAY_TYPENAME_SLUG},
            {type::Descriptor::eMatrix, rw::json::constantes::MATRIX_TYPENAME_SLUG},
            {type::Descriptor::eImage, rw::json::constantes::IMAGE_TYPENAME_SLUG},
            {type::Descriptor::ePointCloud, rw::json::constantes::POINT_CLOUD_TYPENAME_SLUG},
            {type::Descriptor::eQuaternion, rw::json::constantes::QUATERNION_TYPENAME_SLUG},
            {type::Descriptor::ePose, rw::json::constantes::POSE_TYPENAME_SLUG},
            {type::Descriptor::ePosition, rw::json::constantes::POSITION_TYPENAME_SLUG},
            {type::Descriptor::eOrientation, rw::json::constantes::ORIENTATION_TYPENAME_SLUG},
            {type::Descriptor::eIntEnum, rw::json::constantes::INT_ENUM_TYPENAME_SLUG},
            {type::Descriptor::eInt, rw::json::constantes::INT_TYPENAME_SLUG},
            {type::Descriptor::eLong, rw::json::constantes::LONG_TYPENAME_SLUG},
            {type::Descriptor::eFloat, rw::json::constantes::FLOAT_TYPENAME_SLUG},
            {type::Descriptor::eDouble, rw::json::constantes::DOUBLE_TYPENAME_SLUG},
            {type::Descriptor::eBool, rw::json::constantes::BOOL_TYPENAME_SLUG},
            {type::Descriptor::eString, rw::json::constantes::STRING_TYPENAME_SLUG},
            {type::Descriptor::eTime, rw::json::constantes::TIME_TYPENAME_SLUG},
            {type::Descriptor::eAnyObject, rw::json::constantes::ANY_OBJECT_TYPENAME_SLUG}
        };
        const auto String2Descriptor = aron::conversion::util::InvertMap(Descriptor2String);

        const std::map<type::Maybe, std::string> Maybe2String =
        {
            {type::Maybe::eNone, "type::maybe::none"},
            {type::Maybe::eOptional, "type::maybe::optional"},
            {type::Maybe::eRawPointer, "type::maybe::raw_ptr"},
            {type::Maybe::eSharedPointer, "type::maybe::shared_ptr"},
            {type::Maybe::eUniquePointer, "type::maybe::unique_ptr"}
        };
        const auto String2Maybe = aron::conversion::util::InvertMap(Maybe2String);

        const std::map<type::ndarray::ElementType, std::string> NDArrayType2String =
        {
            {type::ndarray::ElementType::int8, "type::ndarray::int8"},
            {type::ndarray::ElementType::int16, "type::ndarray::int16"},
            {type::ndarray::ElementType::int32, "type::ndarray::int32"},
            {type::ndarray::ElementType::uint8, "type::ndarray::uint8"},
            {type::ndarray::ElementType::uint16, "type::ndarray::uint16"},
            {type::ndarray::ElementType::uint32, "type::ndarray::uint32"},
            {type::ndarray::ElementType::float32, "type::ndarray::float32"},
            {type::ndarray::ElementType::float64, "type::ndarray::float64"}
        };
        const auto String2NDArrayType = aron::conversion::util::InvertMap(NDArrayType2String);

        const std::map<type::matrix::ElementType, std::string> MatrixType2String =
        {
            {type::matrix::ElementType::int16, "type::matrix::int16"},
            {type::matrix::ElementType::int32, "type::matrix::int32"},
            {type::matrix::ElementType::int64, "type::matrix::int64"},
            {type::matrix::ElementType::float32, "type::matrix::float32"},
            {type::matrix::ElementType::float64, "type::matrix::float64"}
        };
        const auto String2MatrixType = aron::conversion::util::InvertMap(MatrixType2String);

        const std::map<type::quaternion::ElementType, std::string> QuaternionType2String =
        {
            {type::quaternion::ElementType::float32, "type::quaternion::float32"},
            {type::quaternion::ElementType::float64, "type::quaternion::float64"}
        };
        const auto String2QuaternionType = aron::conversion::util::InvertMap(QuaternionType2String);

        const std::map<type::image::PixelType, std::string> PixelType2String =
        {
            {type::image::PixelType::rgb24, "type::image::rgb24"},
            {type::image::PixelType::depth32, "type::image::depth32"}
        };
        const auto String2PixelType = aron::conversion::util::InvertMap(PixelType2String);

        const std::map<type::pointcloud::VoxelType, std::string> VoxelType2String =
        {
            {type::pointcloud::VoxelType::PointXYZ, "type::pointcloud::pointxyz"},
            {type::pointcloud::VoxelType::PointXYZI, "type::pointcloud::pointxyzi"},
            {type::pointcloud::VoxelType::PointXYZL, "type::pointcloud::pointxyzl"},
            {type::pointcloud::VoxelType::PointXYZRGB, "type::pointcloud::pointxyzrgb"},
            {type::pointcloud::VoxelType::PointXYZRGBA, "type::pointcloud::pointxyzrgba"},
            {type::pointcloud::VoxelType::PointXYZRGBL, "type::pointcloud::pointxyzrgbl"},
            {type::pointcloud::VoxelType::PointXYZHSV, "type::pointcloud::pointxyzhsv"}
        };
        const auto String2VoxelType = aron::conversion::util::InvertMap(VoxelType2String);
    }
}
