#pragma once

#include <shared_mutex>
#include <queue>
#include <thread>
#include <functional>

#include <ArmarXCore/core/ComponentPlugin.h>
#include <ArmarXCore/core/ManagedIceObject.h>

#include <ArmarXCore/core/services/tasks/RunningTask.h>

#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include "helper/LambdaSkillImplementation.h"
#include "detail/SkillImplementationWrapper.h"

namespace armarx::plugins
{
    class SkillProviderComponentPlugin : public ComponentPlugin
    {
    public:
        using ComponentPlugin::ComponentPlugin;

        void preOnInitComponent() override;

        void preOnConnectComponent() override;
        void postOnConnectComponent() override;

        void postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties) override;

        void preOnDisconnectComponent() override;

    private:
        skills::manager::dti::SkillManagerInterfacePrx ownedBySkillManager;
        skills::provider::dti::SkillProviderInterfacePrx myPrx;
    };
}

namespace armarx
{
    class SkillProviderComponentPluginUser :
            virtual public ManagedIceObject,
            virtual public skills::provider::dti::SkillProviderInterface
    {
    public:
        SkillProviderComponentPluginUser();

        skills::provider::dto::SkillDescriptionMap getSkills(const Ice::Current &current = Ice::Current()) override;
        skills::provider::dto::SkillStatusUpdate getSkillExecutionStatus(const std::string& skill, const Ice::Current &current = Ice::Current()) override;
        void executeSkill(const skills::provider::dto::SkillExecutionInfo& executionInfo, const Ice::Current &current = Ice::Current()) override;
        void abortSkill(const std::string &skill, bool waitUntilSkillFinished, const Ice::Current &current = Ice::Current()) override;

    protected:
        void addSkill(const skills::helper::LambdaSkill::FunT&, const skills::SkillDescription&);
        void addSkill(std::unique_ptr<skills::Skill>&&);

    private:
        armarx::plugins::SkillProviderComponentPlugin* plugin = nullptr;

    protected:
        mutable std::shared_mutex skillsMutex;

    public:
        bool connected = false;
        std::map<std::string, skills::detail::SkillImplementationWrapper> skillImplementations;
    };
}
